"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const internal_1 = require("@fluidframework/telemetry-utils/internal");
const internal_2 = require("@fluidframework/test-runtime-utils/internal");
const containerRuntime_js_1 = require("../containerRuntime.js");
function setNavigator(navigator) {
    global.navigator = navigator;
}
describe("Hardware Stats", () => {
    let mockLogger = new internal_1.MockLogger();
    let mockContext = {
        deltaManager: new internal_2.MockDeltaManager(),
        audience: new internal_2.MockAudience(),
        quorum: new internal_2.MockQuorumClients(),
        taggedLogger: mockLogger,
        clientDetails: { capabilities: { interactive: true } },
        updateDirtyContainerState: (dirty) => { },
        getLoadedFromVersion: () => undefined,
    };
    const getDeviceSpecEvents = () => mockLogger.events.filter((event) => event.eventName === "DeviceSpec");
    const loadContainer = async () => containerRuntime_js_1.ContainerRuntime.loadRuntime({
        context: mockContext,
        registryEntries: [],
        runtimeOptions: {
            summaryOptions: {
                summaryConfigOverrides: { state: "disabled" },
            },
        },
        provideEntryPoint: async () => ({
            myProp: "myValue",
        }),
        existing: false,
    });
    beforeEach(async () => {
        mockLogger = new internal_1.MockLogger();
        mockContext = {
            deltaManager: new internal_2.MockDeltaManager(),
            audience: new internal_2.MockAudience(),
            quorum: new internal_2.MockQuorumClients(),
            taggedLogger: mockLogger,
            clientDetails: { capabilities: { interactive: true } },
            updateDirtyContainerState: (dirty) => { },
            getLoadedFromVersion: () => undefined,
        };
    });
    it("should generate correct hardware stats with regular navigator", async () => {
        const navigator = {
            deviceMemory: 10,
            hardwareConcurrency: 8,
        };
        setNavigator(navigator);
        // testing function
        const { deviceMemory, hardwareConcurrency } = (0, containerRuntime_js_1.getDeviceSpec)();
        assert_1.default.strictEqual(deviceMemory, 10, "incorrect deviceMemory value");
        assert_1.default.strictEqual(hardwareConcurrency, 8, "incorrect hardwareConcurrency value");
        await loadContainer();
        // checking telemetry
        const events = getDeviceSpecEvents();
        (0, assert_1.default)(events !== undefined, "No deviceSpec event found");
        assert_1.default.strictEqual(events[0].deviceMemory, 10, "incorrect deviceMemory logged");
        assert_1.default.strictEqual(events[0].hardwareConcurrency, 8, "incorrect hardwareConcurrency logged");
    });
    it("should generate correct hardware stats with null navigator", async () => {
        const navigator = null;
        setNavigator(navigator);
        // testing function
        const { deviceMemory, hardwareConcurrency } = (0, containerRuntime_js_1.getDeviceSpec)();
        assert_1.default.strictEqual(deviceMemory, undefined, "incorrect deviceMemory value");
        assert_1.default.strictEqual(hardwareConcurrency, undefined, "incorrect hardwareConcurrency value");
        await loadContainer();
        // checking telemetry
        const events = getDeviceSpecEvents();
        (0, assert_1.default)(events !== undefined, "No deviceSpec event found");
        assert_1.default.strictEqual(events[0].deviceMemory, undefined, "incorrect deviceMemory logged");
        assert_1.default.strictEqual(events[0].hardwareConcurrency, undefined, "incorrect hardwareConcurrency logged");
    });
    it("should generate correct hardware stats with undefined navigator", async () => {
        const navigator = undefined;
        setNavigator(navigator);
        // testing function
        const { deviceMemory, hardwareConcurrency } = (0, containerRuntime_js_1.getDeviceSpec)();
        assert_1.default.strictEqual(deviceMemory, undefined, "incorrect deviceMemory value");
        assert_1.default.strictEqual(hardwareConcurrency, undefined, "incorrect hardwareConcurrency value");
        await loadContainer();
        // checking telemetry
        const events = getDeviceSpecEvents();
        (0, assert_1.default)(events !== undefined, "No deviceSpec event found");
        assert_1.default.strictEqual(events[0].deviceMemory, undefined, "incorrect deviceMemory logged");
        assert_1.default.strictEqual(events[0].hardwareConcurrency, undefined, "incorrect hardwareConcurrency logged");
    });
});
//# sourceMappingURL=hardwareStats.spec.js.map